package com.builder.uk.todotasks;

import java.io.IOException;
import javax.servlet.ServletContext;
import javax.swing.SwingUtilities;
import org.mortbay.http.HttpContext;
import org.mortbay.http.HttpServer;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.servlet.ServletHandler;
import org.mortbay.jetty.servlet.WebApplicationContext;

/**
 * The ToDoTasks Controller, a simple controller class which creates a Tasks and
 * TasksModel instance, then creates the TasksFrame and awaits user activity.
 * @author Dj Walker-Morgan
 */

public class Controller {
    Tasks tasks;
    TasksModel tasksModel;
    TasksFrame tasksFrame;
    
    /** Creates a new instance of Controller */
    
    public Controller() {
        // Create the tasks and tasksModel
        try {
            tasks=TasksFactory.createTasks(TasksFactory.TasksType.JDBM);
            tasksModel=new TasksModel(tasks);
        } catch (IOException ex) {
            ex.printStackTrace();
            System.exit(1);
        }
        
        // In the event despatch thread, create a tasks frame and make it visible
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                tasksFrame=new TasksFrame(Controller.this);
                tasksFrame.setNotesEnabled(false);
                tasksFrame.setVisible(true);
            }
        });
        
        try {
            Server server=new Server();
            server.addListener(":8000");
            WebApplicationContext context=server.addWebApplication("/","./webapp/");

//   Comment out the previous line and uncomment this line to use a war file
//           WebApplicationContext context=server.addWebApplication("/","./webapp.war");

            context.setAttribute("tasks",tasks);

//   Comment the four lines above and uncomment the lines below to use the hard coded configuration.
//            HttpServer server=new HttpServer();
//            server.addListener(":8000");
//            HttpContext context=server.getContext("/fred/");
//            ServletHandler handler=new ServletHandler();
//            handler.addServlet("Report","/report", "com.builder.uk.todotasks.servlets.Report");
//            context.addHandler(handler);
//            ServletContext sc=handler.getServletContext();
//            sc.setAttribute("tasks",tasks);
            
            try {
                server.start();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        
    }
    
    Long selectedTaskId=null;
    
    
    /**
     * Get the current/active TaskModel
     * @return returns a TaskModel
     */
    public TasksModel getTasksModel() {
        return tasksModel;
    }
    
    
    /**
     * Creates a Task in the Tasks instance
     */
    public void createTask() {
        try {
            tasks.createTask();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     * Removes a Task from the Tasks instance
     * @param id The identifier associated with the Task to be removed
     */
    public void removeTask(Long id) {
        try {
            tasks.removeTask(id);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     * Called when the user selects a Task in the UI
     * @param id The id of the Task selected, or null if no selection is made.
     */
    public void taskSelected(Long id) {
        // If we have a task already selected, look to wether we need to update the notes
        if(selectedTaskId!=null) {
            // Retrieve the Task for the last selected item
            Task lasttask=null;
            try {
                lasttask = tasks.getTask(selectedTaskId);
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            
            // If it still exists...
            if(lasttask!=null) {
                // Get the notes...
                String notes=tasksFrame.getNotes();
                // And if they are different
                if(!notes.equals(lasttask.getNotes())) {
                    // Update the task and write it out
                    lasttask.setNotes(notes);
                    try {
                        tasks.update(lasttask);
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        }
        
        // Update the selected task id
        selectedTaskId=id;
        
        // If it is null, clear the notes and disable editing and return
        if(selectedTaskId==null) {
            tasksFrame.setNotes("");
            tasksFrame.setNotesEnabled(false);
            return;
        }
        
        // Otherwise, get the task and display the notes for editing.
        Task task=null;
        try {
            task = tasks.getTask(id);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        tasksFrame.setNotes(task.getNotes());
        tasksFrame.setNotesEnabled(true);
    }
    
    /**
     * Exits the application
     */
    public void quit() {
        try {
            // Force a deslection (which should save any note changes)
            taskSelected(null);
            // Close the Tasks
            tasks.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        // And leave
        System.exit(0);
    }
}
